"""
3 IBSS, multi-flow experiment

Flow Legend:

Flow 1) IBSS 1    ->  IBSS 2
Flow 2) IBSS 1    ->  IBSS 3
Flow 3) IBSS 2    ->  IBSS 1
Flow 4) IBSS 2    ->  IBSS 3
Flow 5) IBSS 3    ->  IBSS 1
Flow 6) IBSS 3    ->  IBSS 2

Experiment Legend:

Exp 0)  Flow 1 + FLow 2 + Flow 3 + Flow 4
Exp 1)  Flow 1
Exp 2)  Flow 2
Exp 3)  Flow 3
Exp 4)  Flow 4
Exp 5)  Flow 4
Exp 6)  Flow 4

10/23/14 - CRH - updated to 1.0 syntax
9/24/14 - CRH - extended from AP/STA multi-flow app note

"""
import sys
import time
import wlan_exp.config as wlan_exp_config
import wlan_exp.util as wlan_exp_util
import wlan_exp.ltg as wlan_exp_ltg
import wlan_exp.log.util_hdf as hdf_util


#-------------------------------------------------------------------------
#  Global experiment variables
#

# NOTE: change these values to match your experiment setup
NETWORK           = '10.0.0.250' 
NODE_SERIAL_LIST  = ['W3-a-00381', 'W3-a-00021', 'W3-a-00006']

# Set the per-trial duration (in seconds)
TRIAL_TIME = 300

# Set the packet length of each LTG transmission
PACKET_LEN = 1400

# Set the channel for all nodes
CHANNEL = 1

# Set the transmission power in dBm
TX_POW = 5

NETWORK_NAME        = 'MULTI-FLOW-TEST'

#-----------------------------------------------------------------------------
# Local Helper Utilities
#-----------------------------------------------------------------------------

def write_log_file(file_name, data_buffer):
    """Writes log data to a HDF5 file."""
    try:        
        # Get the byte log_data out of the WARPNet buffer
        data = data_buffer.get_bytes()
        
        # Write the byte Log_data to the file 
        hdf_util.log_data_to_hdf5(data, file_name, overwrite=True)
    except AttributeError as err:
        print("Error writing log file: {0}".format(err))
        
        
def reset_everything():    
    wlan_exp_util.broadcast_cmd_set_time(0.0, network_config)     
     
    # Put each node in a known, good state
    for node in nodes:
        node.set_tx_power(TX_POW)
        node.log_configure(log_full_payloads=False)
        node.reset_all()

    # Create a new IBSS network and place all 3 nodes on it.
    bssid = wlan_exp_util.create_locally_administered_bssid(n_ibss_1)
    bss_info = wlan_exp_util.create_bss_info(bssid=bssid, ssid=NETWORK_NAME, channel=CHANNEL)

    n_ibss_1.join(bss_info)
    n_ibss_2.join(bss_info)
    n_ibss_3.join(bss_info)
        
    
def print_and_log(string, file_name):
    f = open(file_name,'a');
    f.write(string)
    f.write("\n")
    f.close()
    print(string)
    


#-------------------------------------------------------------------------
#  Initialization
#
print("\nInitializing experiment\n")

# Create an object that describes the configuration of the host PC
network_config = wlan_exp_config.WlanExpNetworkConfiguration(network=NETWORK) 

# Create an object that describes the WARP v3 nodes that will be used in this experiment
nodes_config   = wlan_exp_config.WlanExpNodesConfiguration(network_config=network_config, 
                                                           serial_numbers=NODE_SERIAL_LIST) 

# Initialize the Nodes
#  This command will fail if either WARP v3 node does not respond
nodes = wlan_exp_util.init_nodes(nodes_config, network_config)

# Extract the AP and STA nodes from the list of initialized nodes
n_ibss_l  = wlan_exp_util.filter_nodes(nodes, mac_high='IBSS')

# Check that we have three IBSS nodes
if (len(n_ibss_l) == 3):
    n_ibss_1 = n_ibss_l[0]
    n_ibss_2 = n_ibss_l[1]
    n_ibss_3 = n_ibss_l[2]    

else:
    print("ERROR: Node configurations did not match requirements of script.\n")
    print(" Ensure two nodes are ready, one using the AP design, one using the STA design\n")
    sys.exit(0)




#%%
#-------------------------------------------------------------------------
#  Run Experiments
#
print("\nRun Experiment:")


#for rate_idx in range(8): #Loop through all 8 supported PHY rates
for rate_idx in [3]: #Test only 1 PHY rate

    rate = wlan_exp_util.wlan_rates[rate_idx] 
    n_ibss_1.set_tx_rate_unicast(rate, curr_assoc=True, new_assoc=True)    
    n_ibss_2.set_tx_rate_unicast(rate, curr_assoc=True, new_assoc=True)    
    n_ibss_3.set_tx_rate_unicast(rate, curr_assoc=True, new_assoc=True)    
    
    #for exp_idx in range(5):  
    for exp_idx in [0]:             
        print("\nExperiment {0}:".format(exp_idx))
        
        reset_everything()
        
        time.sleep(2)

        if exp_idx == 0:        
            flow_1_ltg_id  = n_ibss_1.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_2.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            flow_2_ltg_id  = n_ibss_1.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_3.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)            
            flow_3_ltg_id  = n_ibss_2.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_1.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            flow_4_ltg_id  = n_ibss_2.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_3.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)                        
            flow_5_ltg_id  = n_ibss_3.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_1.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            flow_6_ltg_id  = n_ibss_3.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_2.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)                                    
        elif exp_idx == 1:        
            flow_1_ltg_id  = n_ibss_1.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_2.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            #flow_2_ltg_id  = n_ibss_1.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_3.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)            
            #flow_3_ltg_id  = n_ibss_2.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_1.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            #flow_4_ltg_id  = n_ibss_2.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_3.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)                        
            #flow_5_ltg_id  = n_ibss_3.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_1.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            #flow_6_ltg_id  = n_ibss_3.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_2.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)                                    
        elif exp_idx == 2:        
            #flow_1_ltg_id  = n_ibss_1.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_2.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            flow_2_ltg_id  = n_ibss_1.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_3.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)            
            #flow_3_ltg_id  = n_ibss_2.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_1.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            #flow_4_ltg_id  = n_ibss_2.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_3.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)                        
            #flow_5_ltg_id  = n_ibss_3.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_1.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            #flow_6_ltg_id  = n_ibss_3.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_2.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)                                    
        elif exp_idx == 3:        
            #flow_1_ltg_id  = n_ibss_1.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_2.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            #flow_2_ltg_id  = n_ibss_1.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_3.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)            
            flow_3_ltg_id  = n_ibss_2.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_1.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            #flow_4_ltg_id  = n_ibss_2.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_3.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)                        
            #flow_5_ltg_id  = n_ibss_3.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_1.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            #flow_6_ltg_id  = n_ibss_3.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_2.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)                                    
        elif exp_idx == 4:        
            #flow_1_ltg_id  = n_ibss_1.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_2.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            #flow_2_ltg_id  = n_ibss_1.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_3.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)            
            #flow_3_ltg_id  = n_ibss_2.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_1.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            flow_4_ltg_id  = n_ibss_2.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_3.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)                        
            #flow_5_ltg_id  = n_ibss_3.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_1.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            #flow_6_ltg_id  = n_ibss_3.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_2.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)                                    
        elif exp_idx == 5:        
            #flow_1_ltg_id  = n_ibss_1.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_2.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            #flow_2_ltg_id  = n_ibss_1.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_3.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)            
            #flow_3_ltg_id  = n_ibss_2.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_1.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            #flow_4_ltg_id  = n_ibss_2.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_3.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)                        
            flow_5_ltg_id  = n_ibss_3.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_1.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            #flow_6_ltg_id  = n_ibss_3.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_2.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)                                    
        elif exp_idx == 6:        
            #flow_1_ltg_id  = n_ibss_1.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_2.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            #flow_2_ltg_id  = n_ibss_1.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_3.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)            
            #flow_3_ltg_id  = n_ibss_2.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_1.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            #flow_4_ltg_id  = n_ibss_2.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_3.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)                        
            #flow_5_ltg_id  = n_ibss_3.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_1.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)
            flow_6_ltg_id  = n_ibss_3.ltg_configure(wlan_exp_ltg.FlowConfigCBR(n_ibss_2.wlan_mac_address, PACKET_LEN, 0, 0), auto_start=True)                                                
        
        # This short sleep is to ensure that the LTGs have all started and the
        # below stat structs are created. Otherwise, the returned stats can be
        # NULL. Without proper handling of the NULL case, this can leaad to 
        # errors.
        time.sleep(1)        
        
        # Record the initial Tx/Rx stats
        flow_1_rx_stats_start = n_ibss_2.stats_get_txrx(n_ibss_1)
        flow_2_rx_stats_start = n_ibss_3.stats_get_txrx(n_ibss_1)
        flow_3_rx_stats_start = n_ibss_1.stats_get_txrx(n_ibss_2)
        flow_4_rx_stats_start = n_ibss_3.stats_get_txrx(n_ibss_2)
        flow_5_rx_stats_start = n_ibss_1.stats_get_txrx(n_ibss_3)
        flow_6_rx_stats_start = n_ibss_2.stats_get_txrx(n_ibss_3)        
                
        # Wait for the TRIAL_TIME
        time.sleep(TRIAL_TIME)
        
        # Record the ending Tx/Rx stats
        flow_1_rx_stats_end = n_ibss_2.stats_get_txrx(n_ibss_1)
        flow_2_rx_stats_end = n_ibss_3.stats_get_txrx(n_ibss_1)
        flow_3_rx_stats_end = n_ibss_1.stats_get_txrx(n_ibss_2)
        flow_4_rx_stats_end = n_ibss_3.stats_get_txrx(n_ibss_2)
        flow_5_rx_stats_end = n_ibss_1.stats_get_txrx(n_ibss_3)
        flow_6_rx_stats_end = n_ibss_2.stats_get_txrx(n_ibss_3)     
        
        # Object duplication just so the naming convention is less confusing
        flow_1_tx_stats_start = flow_3_rx_stats_start
        flow_2_tx_stats_start = flow_5_rx_stats_start
        flow_3_tx_stats_start = flow_1_rx_stats_start
        flow_4_tx_stats_start = flow_6_rx_stats_start 
        flow_5_tx_stats_start = flow_2_rx_stats_start
        flow_6_tx_stats_start = flow_4_rx_stats_start

        flow_1_tx_stats_end = flow_3_rx_stats_end
        flow_2_tx_stats_end = flow_5_rx_stats_end
        flow_3_tx_stats_end = flow_1_rx_stats_end
        flow_4_tx_stats_end = flow_6_rx_stats_end 
        flow_5_tx_stats_end = flow_2_rx_stats_end
        flow_6_tx_stats_end = flow_4_rx_stats_end        
        
        # Stop all LTG traffic at each node
        n_ibss_1.ltg_stop_all()
        n_ibss_2.ltg_stop_all()
        n_ibss_3.ltg_stop_all()
        
        n_ibss_1.ltg_remove_all()
        n_ibss_2.ltg_remove_all()
        n_ibss_3.ltg_remove_all()
        
        n_ibss_1.queue_tx_data_purge_all()
        n_ibss_2.queue_tx_data_purge_all()
        n_ibss_3.queue_tx_data_purge_all()
        
        time.sleep(2)        
        
        write_log_file("E{0}_R{1:>.0f}_n_ibss_1.hdf5".format(exp_idx, rate['rate']), n_ibss_1.log_get_all_new(log_tail_pad=0))
        write_log_file("E{0}_R{1:>.0f}_n_ibss_2.hdf5".format(exp_idx, rate['rate']), n_ibss_2.log_get_all_new(log_tail_pad=0))
        write_log_file("E{0}_R{1:>.0f}_n_ibss_3.hdf5".format(exp_idx, rate['rate']), n_ibss_3.log_get_all_new(log_tail_pad=0))    
        
        # Compute the throughput
        print_and_log("     Rate = {0} Mbps -- Flow 1:".format(wlan_exp_util.tx_rate_to_str(rate)),
                      "E{0}_results.txt".format(exp_idx))
        flow_1_num_bytes   = float((flow_1_rx_stats_end['data_num_rx_bytes'] - flow_1_rx_stats_start['data_num_rx_bytes']) * 8)
        flow_1_time_span   = float(flow_1_rx_stats_end['timestamp'] - flow_1_rx_stats_start['timestamp'])
        flow_1_xput        = flow_1_num_bytes / flow_1_time_span       
        
        print_and_log("         Throughput = {0:>5.2f} Mbps".format(flow_1_xput),
                      "E{0}_results.txt".format(exp_idx))

        print_and_log("     Rate = {0} Mbps -- Flow 2:".format(wlan_exp_util.tx_rate_to_str(rate)),
                      "E{0}_results.txt".format(exp_idx))
        flow_2_num_bytes   = float((flow_2_rx_stats_end['data_num_rx_bytes'] - flow_2_rx_stats_start['data_num_rx_bytes']) * 8)
        flow_2_time_span   = float(flow_2_rx_stats_end['timestamp'] - flow_2_rx_stats_start['timestamp'])
        flow_2_xput        = flow_2_num_bytes / flow_2_time_span

        print_and_log("         Throughput = {0:>5.2f} Mbps".format(flow_2_xput),
                      "E{0}_results.txt".format(exp_idx))
        
        print_and_log("     Rate = {0} Mbps -- Flow 3:".format(wlan_exp_util.tx_rate_to_str(rate)),
                      "E{0}_results.txt".format(exp_idx))
        flow_3_num_bytes   = float((flow_3_rx_stats_end['data_num_rx_bytes'] - flow_3_rx_stats_start['data_num_rx_bytes']) * 8)
        flow_3_time_span   = float(flow_3_rx_stats_end['timestamp'] - flow_3_rx_stats_start['timestamp'])
        flow_3_xput        = flow_3_num_bytes / flow_3_time_span
            
        print_and_log("         Throughput = {0:>5.2f} Mbps".format(flow_3_xput),
                      "E{0}_results.txt".format(exp_idx))
        
        print_and_log("     Rate = {0} Mbps -- Flow 4:".format(wlan_exp_util.tx_rate_to_str(rate)),
                      "E{0}_results.txt".format(exp_idx))
        flow_4_num_bytes   = float((flow_4_rx_stats_end['data_num_rx_bytes'] - flow_4_rx_stats_start['data_num_rx_bytes']) * 8)
        flow_4_time_span   = float(flow_4_rx_stats_end['timestamp'] - flow_4_rx_stats_start['timestamp'])
        flow_4_xput        = flow_4_num_bytes / flow_4_time_span
 
        print_and_log("         Throughput = {0:>5.2f} Mbps".format(flow_4_xput),
                      "E{0}_results.txt".format(exp_idx))
                      
        print_and_log("     Rate = {0} Mbps -- Flow 5:".format(wlan_exp_util.tx_rate_to_str(rate)),
                      "E{0}_results.txt".format(exp_idx))
        flow_5_num_bytes   = float((flow_5_rx_stats_end['data_num_rx_bytes'] - flow_5_rx_stats_start['data_num_rx_bytes']) * 8)
        flow_5_time_span   = float(flow_5_rx_stats_end['timestamp'] - flow_5_rx_stats_start['timestamp'])
        flow_5_xput        = flow_5_num_bytes / flow_5_time_span
 
        print_and_log("         Throughput = {0:>5.2f} Mbps".format(flow_5_xput),
                      "E{0}_results.txt".format(exp_idx))
                      
        print_and_log("     Rate = {0} Mbps -- Flow 6:".format(wlan_exp_util.tx_rate_to_str(rate)),
                      "E{0}_results.txt".format(exp_idx))
        flow_6_num_bytes   = float((flow_6_rx_stats_end['data_num_rx_bytes'] - flow_6_rx_stats_start['data_num_rx_bytes']) * 8)
        flow_6_time_span   = float(flow_6_rx_stats_end['timestamp'] - flow_6_rx_stats_start['timestamp'])
        flow_6_xput        = flow_6_num_bytes / flow_6_time_span
 
        print_and_log("         Throughput = {0:>5.2f} Mbps".format(flow_6_xput),
                      "E{0}_results.txt".format(exp_idx))                      

        
        
